(*----------------------------------------------------------------------------*
 *  Direct3D sample from DirectX 9.0 SDK Summer 2003                          *
 *  Delphi adaptation by Alexey Barkovoy (e-mail: clootie@ixbt.com)           *
 *                                                                            *
 *  Delphi versions 5-7 are supported                                         *
 *                                                                            *
 *  Modified: 28-Jan-2004                                                     *
 *                                                                            *
 *  Latest version can be downloaded from:                                    *
 *     http://clootie.narod.ru/delphi                                         *
 *----------------------------------------------------------------------------*)
//-----------------------------------------------------------------------------
// File: BasicHLSL.cpp
//
// Desc: This sample shows a simple example of the Microsoft Direct3D's High-Level
//       Shader Language (HLSL) using the Effect interface.
//
// Copyright (c) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------

unit BasicHLSLUnit;

interface

uses
  Windows, Messages,
  Direct3D9, D3DX9, dxerr9,
  D3DApp, D3DFont, D3DUtil, DXUtil,
  Resource;

// C++Builder compile hack
{$NOINCLUDE Resource}
{$HPPEMIT '#include "Resource.hpp"'}

{.$DEFINE DEBUG_VS}   // Uncomment this line to debug vertex shaders
{.$DEFINE DEBUG_PS}   // Uncomment this line to debug pixel shaders

type
  //-----------------------------------------------------------------------------
  // Name: class CMyD3DApplication
  // Desc: Main class to run this application. Most functionality is inherited
  //       from the CD3DApplication base class.
  //-----------------------------------------------------------------------------
  CMyD3DApplication = class(CD3DApplication)
  private
    m_pFont: ID3DXFont;                // Font for drawing text
    m_bShowHelp: Boolean;              // If true, it renders the UI control text
    m_Camera: CModelViewerCamera;      // A model viewing camera
    m_pEffect: ID3DXEffect;            // D3DX effect interface

    m_pMesh:                 ID3DXMesh;              // Mesh object
    m_pMeshTexture:          IDirect3DTexture9;      // Mesh texture

    m_hTime:                 TD3DXHandle;  // Handle to var in the effect
    m_hWorld:                TD3DXHandle;  // Handle to var in the effect
    m_hMeshTexture:          TD3DXHandle;  // Handle to var in the effect
    m_hWorldViewProjection:  TD3DXHandle;  // Handle to var in the effect
    m_hTechniqueRenderScene: TD3DXHandle;  // Handle to technique in the effect

  protected
    function OneTimeSceneInit: HRESULT; override;
    function InitDeviceObjects: HRESULT; override;
    function RestoreDeviceObjects: HRESULT; override;
    function InvalidateDeviceObjects: HRESULT; override;
    function DeleteDeviceObjects: HRESULT; override;
    function FinalCleanup: HRESULT; override;
    function Render: HRESULT; override;
    function FrameMove: HRESULT; override;
    function ConfirmDevice(const pCaps: TD3DCaps9; dwBehavior: DWORD;
      adapterFormat: TD3DFormat; backBufferFormat: TD3DFormat): HRESULT; override;
    function LoadMesh(strFileName: PChar; out ppMesh: ID3DXMesh): HRESULT;
    function RenderText: HRESULT;

  public
    constructor Create;
    function MsgProc(hWnd: HWND; uMsg: LongWord; wParam: WPARAM; lParam: LPARAM): LRESULT; override;
  end;



implementation

uses SysUtils;

const
  UnitName = 'BasicHLSL.pas';

{ CMyD3DApplication }

//-----------------------------------------------------------------------------
// Name: CMyD3DApplication()
// Desc: Application constructor.   Paired with ~CMyD3DApplication()
//       Member variables should be initialized to a known state here.  
//       The application window has not yet been created and no Direct3D device 
//       has been created, so any initialization that depends on a window or 
//       Direct3D should be deferred to a later stage. 
//-----------------------------------------------------------------------------
constructor CMyD3DApplication.Create;
begin
  inherited Create;
  m_Camera:= CModelViewerCamera.Create;

  m_strWindowTitle := 'BasicHLSL';
  m_d3dEnumeration.AppUsesDepthBuffer := True;
  m_dwCreationWidth  := 640;
  m_dwCreationHeight := 480;
  m_pFont := nil;
  m_bShowHelp := True;
  m_pEffect := nil;

  m_hWorld := nil;
  m_hMeshTexture := nil;
  m_hWorldViewProjection := nil;
  m_hTechniqueRenderScene := nil;

  m_pMesh := nil;
  m_pMeshTexture := nil;
end;


//-----------------------------------------------------------------------------
// Name: OneTimeSceneInit()
// Desc: Paired with FinalCleanup().
//       The window has been created and the IDirect3D9 interface has been
//       created, but the device has not been created yet.  Here you can
//       perform application-related initialization and cleanup that does
//       not depend on a device.
//-----------------------------------------------------------------------------
function CMyD3DApplication.OneTimeSceneInit: HRESULT;
begin
  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: InitDeviceObjects()
// Desc: Paired with DeleteDeviceObjects()
//       The device has been created.  Resources that are not lost on
//       Reset() can be created here -- resources in D3DPOOL_MANAGED,
//       D3DPOOL_SCRATCH, or D3DPOOL_SYSTEMMEM.  Image surfaces created via
//       CreateImageSurface are never lost and can be created here.  Vertex
//       shaders and pixel shaders can also be created here as they are not
//       lost on Reset().
//-----------------------------------------------------------------------------
function CMyD3DApplication.InitDeviceObjects: HRESULT;
var
  hDC: Windows.HDC;
  nHeight: Integer;
  dwShaderFlags: DWORD;
  str: array[0..MAX_PATH-1] of Char;
  fAspectRatio: Single; 
  vecEye: TD3DXVector3;
  vecAt: TD3DXVector3;
begin
  // Initialize the font
  hDC := GetDC(0);
  nHeight := -MulDiv(9, GetDeviceCaps(hDC, LOGPIXELSY), 72);
  ReleaseDC(0, hDC);
  Result := D3DXCreateFont(m_pd3dDevice, nHeight, 0, FW_BOLD, 0, False, DEFAULT_CHARSET,
                           OUT_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH or FF_DONTCARE,
                           'Arial', m_pFont);
  if FAILED(Result) then
  begin
    Result:= DXTRACE_ERR('D3DXCreateFont', Result, UnitName);
    Exit;
  end;

  // Load the mesh
  Result := LoadMesh('tiger.x', m_pMesh);
  if FAILED(Result) then
  begin
    Result:= DXTRACE_ERR('LoadMesh', Result, UnitName);
    Exit;
  end;

  // Define DEBUG_VS and/or DEBUG_PS to debug vertex and/or pixel shaders with the shader debugger.
  // Debugging vertex shaders requires either REF or software vertex processing, and debugging
  // pixel shaders requires REF.  The D3DXSHADER_FORCE_*_SOFTWARE_NOOPT flag improves the debug
  // experience in the shader debugger.  It enables source level debugging, prevents instruction
  // reordering, prevents dead code elimination, and forces the compiler to compile against the next
  // higher available software target, which ensures that the unoptimized shaders do not exceed
  // the shader model limitations.  Setting these flags will cause slower rendering since the shaders
  // will be unoptimized and forced into software.  See the DirectX documentation for more information
  // about using the shader debugger.
  dwShaderFlags := 0;
  {$IFDEF DEBUG_VS}
    dwShaderFlags := dwShaderFlags or D3DXSHADER_FORCE_VS_SOFTWARE_NOOPT;
  {$ENDIF}
  {$IFDEF DEBUG_PS}
    dwShaderFlags := dwShaderFlags or D3DXSHADER_FORCE_PS_SOFTWARE_NOOPT;
  {$ENDIF}

  // Read the D3DX effect file
  Result := DXUtil_FindMediaFileCb(str, SizeOf(str), 'BasicHLSL.fx');
  if FAILED(Result) then
  begin
    Result:= DXTRACE_ERR('DXUtil_FindMediaFileCb', D3DAPPERR_MEDIANOTFOUND, UnitName);
    Exit;
  end;
  Result := D3DXCreateEffectFromFile(m_pd3dDevice, str, nil, nil, dwShaderFlags, nil, m_pEffect, nil);

  // If this fails, there should be debug output as to
  // they the .fx file failed to compile
  if FAILED(Result) then
  begin
    Result:= DXTRACE_ERR('D3DXCreateEffectFromFile', Result, UnitName);
    Exit;
  end;

  // Setup the camera's projection parameters
  fAspectRatio := m_d3dsdBackBuffer.Width / m_d3dsdBackBuffer.Height;
  m_Camera.SetProjParams(D3DX_PI/4, fAspectRatio, 0.1, 1000.0);
  m_Camera.SetWindow(m_d3dsdBackBuffer.Width, m_d3dsdBackBuffer.Height);

  // Setup the camera's view parameters
  vecEye:= D3DXVector3(0.0, 0.0, -5.0);
  vecAt:=  D3DXVector3(0.0, 0.0, -0.0);
  m_Camera.SetViewParams(vecEye, vecAt);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: LoadMesh()
// Desc: Load the mesh and ensure the mesh has normals, and optimize
//       the mesh for this graphics card's vertex cache so the triangle list
//       inside in the mesh will cache miss less which improves perf.
//-----------------------------------------------------------------------------
function CMyD3DApplication.LoadMesh(strFileName: PChar; out ppMesh: ID3DXMesh): HRESULT;
var
  pMesh: ID3DXMesh;
  str: array[0..MAX_PATH-1] of Char;
  rgdwAdjacency: PDWORD;
  pTempMesh: ID3DXMesh;
begin
  if (@ppMesh = nil) then
  begin
    Result:= E_INVALIDARG;
    Exit;
  end;

  // Load the mesh with D3DX and get back a ID3DXMesh*.  For this
  // sample we'll ignore the X file's embedded materials since we know
  // exactly the model we're loading.  See the mesh samples such as
  // "OptimizedMesh" for a more generic mesh loading example.
  Result := DXUtil_FindMediaFileCb(str, SizeOf(str), strFileName);
  if FAILED(Result) then
  begin
    Result:= DXTRACE_ERR('DXUtil_FindMediaFileCb', D3DAPPERR_MEDIANOTFOUND, UnitName);
    Exit;
  end;
  Result := D3DXLoadMeshFromX(str, D3DXMESH_MANAGED,
                              m_pd3dDevice, nil, nil, nil, nil, pMesh);
  if FAILED(Result) or (pMesh = nil) then
  begin
    Result:= DXTRACE_ERR('D3DXLoadMeshFromX', Result, UnitName);
    Exit;
  end;

  // Make sure there are normals which are required for lighting
  if (pMesh.GetFVF and D3DFVF_NORMAL = 0) then
  begin
    Result := pMesh.CloneMeshFVF(pMesh.GetOptions,
                                 pMesh.GetFVF or D3DFVF_NORMAL,
                                 m_pd3dDevice, pTempMesh);
    if FAILED(Result) then
    begin
      Result:= DXTRACE_ERR('CloneMeshFVF', Result, UnitName);
      Exit;
    end;

    D3DXComputeNormals(pTempMesh, nil);

    SAFE_RELEASE(pMesh);
    pMesh := pTempMesh;
  end;

  // Optimize the mesh for this graphics card's vertex cache
  // so when rendering the mesh's triangle list the vertices will
  // cache hit more often so it won't have to re-execute the vertex shader
  // on those vertices so it will improves perf.
  try
    GetMem(rgdwAdjacency, SizeOf(DWORD) * pMesh.GetNumFaces * 3);
  except
    Result:= E_OUTOFMEMORY;
    Exit;
  end;
  pMesh.ConvertPointRepsToAdjacency(nil, rgdwAdjacency);
  pMesh.OptimizeInplace(D3DXMESHOPT_VERTEXCACHE, rgdwAdjacency, nil, nil, nil);
  FreeMem(rgdwAdjacency);

  ppMesh := pMesh;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: RestoreDeviceObjects()
// Desc: Paired with InvalidateDeviceObjects()
//       The device exists, but may have just been Reset().  Resources in
//       D3DPOOL_DEFAULT and any other device state that persists during
//       rendering should be set here.  Render states, matrices, textures,
//       etc., that don't change during rendering can be set once here to
//       avoid redundant state setting during Render() or FrameMove().
//-----------------------------------------------------------------------------
function CMyD3DApplication.RestoreDeviceObjects: HRESULT;
var
  str: array[0..MAX_PATH] of Char;
  colorMtrlDiffuse: TD3DXColor;
  colorMtrlAmbient: TD3DXColor;
begin
  if Assigned(m_pFont) then m_pFont.OnResetDevice;
  if Assigned(m_pEffect) then m_pEffect.OnResetDevice;

  // Create the mesh texture from a file
  Result := DXUtil_FindMediaFileCb(str, SizeOf(str), 'tiger.bmp');
  if FAILED(Result) then
  begin
    Result:= DXTRACE_ERR('DXUtil_FindMediaFileCb', D3DAPPERR_MEDIANOTFOUND, UnitName);
    Exit;
  end;
  Result:=  D3DXCreateTextureFromFileEx(m_pd3dDevice, str, D3DX_DEFAULT, D3DX_DEFAULT,
                                        D3DX_DEFAULT, 0, D3DFMT_UNKNOWN, D3DPOOL_DEFAULT,
                                        D3DX_DEFAULT, D3DX_DEFAULT, 0,
                                        nil, nil, m_pMeshTexture);
  if FAILED(Result) then
  begin
    Result:= DXTRACE_ERR('D3DXCreateTextureFromFileEx', Result, UnitName);
    Exit;
  end;

  // Set effect variables as needed
  colorMtrlDiffuse:= D3DXColor(1.0, 1.0, 1.0, 1.0);
  colorMtrlAmbient:= D3DXColor(0.35, 0.35, 0.35, 0);
  m_pEffect.SetVector('g_MaterialAmbientColor', PD3DXVector4(@colorMtrlAmbient)^);
  m_pEffect.SetVector('g_MaterialDiffuseColor', PD3DXVector4(@colorMtrlDiffuse)^);

  // To read or write to D3DX effect variables we can use the string name 
  // instead of using handles, however it improves perf to use handles since then 
  // D3DX won't have to spend time doing string compares
  m_hTechniqueRenderScene         := m_pEffect.GetTechniqueByName('RenderScene');
  m_hTime                         := m_pEffect.GetParameterByName(nil, 'g_fTime');
  m_hWorld                        := m_pEffect.GetParameterByName(nil, 'g_mWorld');
  m_hWorldViewProjection          := m_pEffect.GetParameterByName(nil, 'g_mWorldViewProjection');
  m_hMeshTexture                  := m_pEffect.GetParameterByName(nil, 'g_MeshTexture');
end;


//-----------------------------------------------------------------------------
// Name: FrameMove()
// Desc: Called once per frame, the call is the entry point for animating
//       the scene.
//-----------------------------------------------------------------------------
function CMyD3DApplication.FrameMove: HRESULT;
begin
  // Update the camera's postion based on user input
  m_Camera.FrameMove(m_fElapsedTime);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Called once per frame, the call is the entry point for 3d
//       rendering. This function sets up render states, clears the
//       viewport, and renders the scene.
//-----------------------------------------------------------------------------
function CMyD3DApplication.Render: HRESULT;
var
  m: TD3DXMatrixA16;
  mWorld: TD3DXMatrixA16;
  mView: TD3DXMatrixA16;
  mProj: TD3DXMatrixA16;
  mWorldViewProjection: TD3DXMatrixA16;
  iPass, cPasses: LongWord;
begin
  // Clear the render target and the zbuffer
  m_pd3dDevice.Clear(0, nil, D3DCLEAR_TARGET or D3DCLEAR_ZBUFFER, $00003F3F, 1.0, 0);

  // Render the scene
  if SUCCEEDED(m_pd3dDevice.BeginScene) then
  begin
    // Set world drawing technique
    m_pEffect.SetTechnique(m_hTechniqueRenderScene);

    // Get the projection & view matrix from the camera class
    mWorld := m_Camera.GetWorldMatrix^;
    mProj := m_Camera.GetProjMatrix^;
    mView := m_Camera.GetViewMatrix^;

    // mWorldViewProjection = mWorld * mView * mProj;
    D3DXMatrixMultiply(m, mView, mProj);
    D3DXMatrixMultiply(mWorldViewProjection, mWorld, m);

    // Update the effect's variables
    m_pEffect.SetMatrix(m_hWorldViewProjection, mWorldViewProjection);
    m_pEffect.SetMatrix(m_hWorld, mWorld);
    m_pEffect.SetFloat(m_hTime, m_fTime);
    m_pEffect.SetTexture(m_hMeshTexture, m_pMeshTexture);

    Result := m_pEffect._Begin(@cPasses, 0);
    if FAILED(Result) then
    begin
      Result:= DXTRACE_ERR('Begin', Result, UnitName);
      Exit;
    end;

    for iPass := 0 to cPasses - 1 do
    begin
      Result := m_pEffect.Pass(iPass);
      if FAILED(Result) then
      begin
        Result:= DXTRACE_ERR('Pass', Result, UnitName);
        Exit;
      end;

      Result := m_pMesh.DrawSubset(0);
      if FAILED(Result) then
      begin
        Result:= DXTRACE_ERR('DrawSubset', Result, UnitName);
        Exit;
      end;
    end;
    m_pEffect._End;

    Result := RenderText;
    if FAILED(Result) then
    begin
      Result:= DXTRACE_ERR('RenderText', Result, UnitName);
      Exit;
    end;

    m_pd3dDevice.EndScene;
  end;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: RenderText()
// Desc: Draw the help & statistics for running sample
//-----------------------------------------------------------------------------
function CMyD3DApplication.RenderText: HRESULT;
var
  rc: TRect;
  iLineY: Integer;
  sz: array[0..99] of Char;
begin
  if (nil = m_pFont) then
  begin
    Result:= S_OK;
    Exit;
  end;

{$IFDEF UNICODE}
  For UNICODE you should use "m_pFont.DrawTextW" routine!!!
{$ENDIF}

  // Output statistics
  iLineY := 0;
  SetRect(rc, 2, iLineY, 0, 0); Inc(iLineY, 15);
  m_pFont.DrawTextA(nil, m_strFrameStats, -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 0.0, 1.0)));
  SetRect(rc, 2, iLineY, 0, 0); // Inc(iLineY, 15); //BAA: never used anyway
  m_pFont.DrawTextA( nil, m_strDeviceStats, -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 0.0, 1.0)));

  // Output statistics
  iLineY := 0;
  SetRect(rc, 2, iLineY, 0, 0); Inc(iLineY, 15);
  Result:= m_pFont.DrawTextA(nil, m_strFrameStats, -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 0.0, 1.0)));
    if FAILED(Result) then begin Result:= DXTRACE_ERR('DrawText', Result, UnitName); Exit; end;
  SetRect(rc, 2, iLineY, 0, 0); Inc(iLineY, 15);
  Result:= m_pFont.DrawTextA(nil, m_strDeviceStats, -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 0.0, 1.0)));
    if FAILED(Result) then begin Result:= DXTRACE_ERR('DrawText', Result, UnitName); Exit; end;

  StrLFmt(sz, 100, 'AppStat1=%0.1f AppStat2=%0.4f', [m_fTime, Sin(m_fTime)]); sz[99] := #0;

  SetRect(rc, 2, iLineY, 0, 0); Inc(iLineY, 15);
  Result := m_pFont.DrawTextA(nil, sz, -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 1.0, 1.0)));
    if FAILED(Result) then begin Result:= DXTRACE_ERR('DrawText', Result, UnitName); Exit; end;

  // Draw help
  if m_bShowHelp then
  begin
    iLineY := m_d3dsdBackBuffer.Height-15*6;
    SetRect(rc, 2, iLineY, 0, 0); Inc(iLineY, 15);
    m_pFont.DrawTextA(nil, 'Controls:', -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 0.75, 0.0, 1.0)));
      if FAILED(Result) then begin Result:= DXTRACE_ERR('DrawText', Result, UnitName); Exit; end;
    SetRect(rc, 20, iLineY, 0, 0);
    m_pFont.DrawTextA(nil, 'Rotate model: Left mouse button'#10 +
                           'Rotate camera: Right mouse button'#10 +
                           'Zoom camera: Mouse wheel scroll'#10 +
                           'Hide help: F1'#10,
                           -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 0.75, 0.0, 1.0)));
    if FAILED(Result) then begin Result:= DXTRACE_ERR('DrawText', Result, UnitName); Exit; end;

    SetRect(rc, 250, iLineY, 0, 0);
    m_pFont.DrawTextA(nil, 'Change Device: F2'#10 +
                           'View readme: F9'#10 +
                           'Quit: ESC',
                           -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 0.75, 0.0, 1.0)));
    if FAILED(Result) then begin Result:= DXTRACE_ERR('DrawText', Result, UnitName); Exit; end;
  end else
  begin
    SetRect(rc, 2, iLineY, 0, 0); // Inc(iLineY, 15); //BAA: never used anyway
    m_pFont.DrawTextA(nil, 'Press F1 for help',
                      -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 1.0, 1.0)));
    if FAILED(Result) then begin Result:= DXTRACE_ERR('DrawText', Result, UnitName); Exit; end;
  end;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: InvalidateDeviceObjects()
// Desc: Invalidates device objects.  Paired with RestoreDeviceObjects()
//-----------------------------------------------------------------------------
function CMyD3DApplication.InvalidateDeviceObjects: HRESULT;
begin
  if Assigned(m_pFont) then m_pFont.OnLostDevice;
  if Assigned(m_pEffect) then  m_pEffect.OnLostDevice;

  SAFE_RELEASE(m_pMeshTexture);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: DeleteDeviceObjects()
// Desc: Paired with InitDeviceObjects()
//       Called when the app is exiting, or the device is being changed,
//       this function deletes any device dependent objects.
//-----------------------------------------------------------------------------
function CMyD3DApplication.DeleteDeviceObjects: HRESULT;
begin
  SAFE_RELEASE(m_pEffect);
  SAFE_RELEASE(m_pFont);
  SAFE_RELEASE(m_pMesh);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: FinalCleanup()
// Desc: Paired with OneTimeSceneInit()
//       Called before the app exits, this function gives the app the chance
//       to cleanup after itself.
//-----------------------------------------------------------------------------
function CMyD3DApplication.FinalCleanup: HRESULT;
begin
  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: ConfirmDevice()
// Desc: Called during device initialization, this code checks the device
//       for some minimum set of capabilities
//-----------------------------------------------------------------------------
function CMyD3DApplication.ConfirmDevice(const pCaps: TD3DCaps9; dwBehavior: DWORD;
  adapterFormat: TD3DFormat; backBufferFormat: TD3DFormat): HRESULT;
begin
  // Debugging vertex shaders requires either REF or software vertex processing
  // and debugging pixel shaders requires REF.
  {$IFDEF DEBUG_VS}
  if (pCaps.DeviceType <> D3DDEVTYPE_REF) and
     ((dwBehavior and D3DCREATE_SOFTWARE_VERTEXPROCESSING) = 0) then
  begin
    Result:= E_FAIL;
    Exit;
  end;
  {$ENDIF}
  {$IFDEF DEBUG_PS}
  if (pCaps.DeviceType <> D3DDEVTYPE_REF)
  begin
    Result:= E_FAIL;
    Exit;
  end;
  {$ENDIF}

  Result:= E_FAIL;

  if (dwBehavior and D3DCREATE_PUREDEVICE <> 0) then Exit;

  // No fallback, so need ps1.1
  if (pCaps.PixelShaderVersion < D3DPS_VERSION(1,1)) then Exit;

  // If device doesn't support 1.1 vertex shaders in HW, switch to SWVP.
  if (pCaps.VertexShaderVersion < D3DVS_VERSION(1,1)) then
  begin
    if ((dwBehavior and D3DCREATE_SOFTWARE_VERTEXPROCESSING) = 0) then Exit;
  end;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: MsgProc()
// Desc: Overrrides the main WndProc, so the sample can do custom message
//       handling (e.g. processing mouse, keyboard, or menu commands).
//-----------------------------------------------------------------------------
function CMyD3DApplication.MsgProc(hWnd: HWND; uMsg: LongWord;
  wParam: WPARAM; lParam: LPARAM): LRESULT;
begin
  m_Camera.HandleMessages(hWnd, uMsg, wParam, lParam);

  case uMsg of
    WM_COMMAND:
    begin
      case LOWORD(wParam) of
        IDM_TOGGLEHELP: m_bShowHelp := not m_bShowHelp;
      end;
    end;
  end;

  Result:= inherited MsgProc(hWnd, uMsg, wParam, lParam);
end;

end.






